@@ -12,17 +12,16 @@ import (
 	. "github.com/onsi/gomega"
 
 	corev1 "k8s.io/api/core/v1"
+	rbacv1 "k8s.io/api/rbac/v1"
 	"k8s.io/apimachinery/pkg/api/errors"
 	"k8s.io/apimachinery/pkg/api/resource"
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
-	"k8s.io/apimachinery/pkg/fields"
 	"k8s.io/apimachinery/pkg/labels"
-	"k8s.io/apimachinery/pkg/runtime"
 	"k8s.io/kubernetes/pkg/kubelet/cm/cpuset"
 	"k8s.io/utils/pointer"
 	goclient "sigs.k8s.io/controller-runtime/pkg/client"
 
-	perfv1alpha1 "github.com/openshift-kni/performance-addon-operators/pkg/apis/performance/v1alpha1"
+	perfv1 "github.com/openshift-kni/performance-addon-operators/pkg/apis/performance/v1"
 	mcv1 "github.com/openshift/machine-config-operator/pkg/apis/machineconfiguration.openshift.io/v1"
 
 	sriovk8sv1 "github.com/openshift/sriov-network-operator/pkg/apis/k8s/v1"
@@ -33,34 +32,39 @@ import (
 	sriovnetwork "github.com/openshift/sriov-network-operator/test/util/network"
 
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/client"
+	"github.com/openshift-kni/cnf-features-deploy/functests/utils/discovery"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/execute"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/images"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/machineconfigpool"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/namespaces"
+	"github.com/openshift-kni/cnf-features-deploy/functests/utils/nodes"
 	"github.com/openshift-kni/cnf-features-deploy/functests/utils/pods"
+	"github.com/openshift-kni/cnf-features-deploy/functests/utils/sriov"
 )
 
 const (
-	SRIOV_OPERATOR_NAMESPACE = "openshift-sriov-network-operator"
-	LOG_ENTRY                = "Accumulated forward statistics for all ports"
-	PCI_ENV_VARIABLE_NAME    = "PCIDEVICE_OPENSHIFT_IO_DPDKNIC"
-	DEMO_APP_NAMESPACE       = "dpdk"
+	LOG_ENTRY              = "Accumulated forward statistics for all ports"
+	DEMO_APP_NAMESPACE     = "dpdk"
+	SERVER_TESTPMD_COMMAND = "testpmd -l ${CPU} -w ${PCIDEVICE_OPENSHIFT_IO_%s} --iova-mode=va -- -i --portmask=0x1 --nb-cores=2 --forward-mode=mac --port-topology=loop --no-mlockall"
+	CLIENT_TESTPMD_COMMAND = "testpmd -l ${CPU} -w ${PCIDEVICE_OPENSHIFT_IO_%s} --iova-mode=va -- -i --portmask=0x1 --nb-cores=2 --eth-peer=0,ff:ff:ff:ff:ff:ff --forward-mode=txonly --no-mlockall"
 )
 
 var (
-	machineConfigPoolName  string
-	performanceProfileName string
+	machineConfigPoolName          string
+	performanceProfileName         string
+	enforcedPerformanceProfileName string
+
+	resourceName = "dpdknic"
 
 	sriovclient *sriovtestclient.ClientSet
 
 	OriginalSriovPolicies      []*sriovv1.SriovNetworkNodePolicy
 	OriginalSriovNetworks      []*sriovv1.SriovNetwork
-	OriginalPerformanceProfile *perfv1alpha1.PerformanceProfile
-	waitingTime                time.Duration = 20 * time.Minute
+	OriginalPerformanceProfile *perfv1.PerformanceProfile
 )
 
 func init() {
-	machineConfigPoolName = os.Getenv("ROLE_WORKER_RT")
+	machineConfigPoolName = os.Getenv("ROLE_WORKER_CNF")
 	if machineConfigPoolName == "" {
 		machineConfigPoolName = "worker-cnf"
 	}
@@ -68,6 +72,8 @@ func init() {
 	performanceProfileName = os.Getenv("PERF_TEST_PROFILE")
 	if performanceProfileName == "" {
 		performanceProfileName = "performance"
+	} else {
+		enforcedPerformanceProfileName = performanceProfileName
 	}
 
 	OriginalSriovPolicies = make([]*sriovv1.SriovNetworkNodePolicy, 0)
@@ -75,19 +81,13 @@ func init() {
 
 	// Reuse the sriov client
 	// Use the SRIOV test client
-	sriovclient = sriovtestclient.New("", func(scheme *runtime.Scheme) {
-		sriovv1.AddToScheme(scheme)
-	})
-
-	waitingEnv := os.Getenv("SRIOV_WAITING_TIME")
-	newTime, err := strconv.Atoi(waitingEnv)
-	if err == nil && newTime != 0 {
-		waitingTime = time.Duration(newTime) * time.Minute
-	}
+	sriovclient = sriovtestclient.New("")
 }
 
 var _ = Describe("dpdk", func() {
 	var dpdkWorkloadPod *corev1.Pod
+	var discoverySuccessful bool
+	discoveryFailedReason := "Can not run tests in discovery mode. Failed to discover required resources"
 
 	execute.BeforeAll(func() {
 		var exist bool
@@ -95,24 +95,64 @@ var _ = Describe("dpdk", func() {
 		if exist {
 			return
 		}
+		nodeSelector, _ := nodes.PodLabelSelector()
 
-		findOrOverridePerformanceProfile()
-		findOrOverrideSriovNetwork()
+		if discovery.Enabled() {
+			var performanceProfiles []*perfv1.PerformanceProfile
+			discoverySuccessful, discoveryFailedReason, performanceProfiles = discoverPerformanceProfiles()
 
-		dpdkWorkloadPod = createPod()
+			if !discoverySuccessful {
+				discoveryFailedReason = "Could not find a valid performance profile"
+				return
+			}
+
+			discovered, err := discovery.DiscoverPerformanceProfileAndPolicyWithAvailableNodes(client.Client, sriovclient, namespaces.SRIOVOperator, resourceName, performanceProfiles, nodeSelector)
+			if err != nil {
+				discoverySuccessful, discoveryFailedReason = false, "Can not run tests in discovery mode. Failed to discover required resources."
+				return
+			}
+			profile, sriovDevice := discovered.Profile, discovered.Device
+			resourceName = discovered.Resource
+
+			nodeSelector = nodes.SelectorUnion(nodeSelector, profile.Spec.NodeSelector)
+			CreateSriovNetwork(sriovDevice, "test-dpdk-network", resourceName)
+
+		} else {
+			findOrOverridePerformanceProfile()
+			findOrOverrideSriovPolicyAndNetwork()
+		}
+
+		var err error
+		dpdkWorkloadPod, err = createDPDKWorkload(nodeSelector,
+			strings.ToUpper(resourceName),
+			fmt.Sprintf(SERVER_TESTPMD_COMMAND, strings.ToUpper(resourceName)),
+			60,
+			true)
+		Expect(err).ToNot(HaveOccurred())
+
+		_, err = createDPDKWorkload(nodeSelector,
+			strings.ToUpper(resourceName),
+			fmt.Sprintf(CLIENT_TESTPMD_COMMAND, strings.ToUpper(resourceName)),
+			10,
+			false)
+		Expect(err).ToNot(HaveOccurred())
 	})
 
-	Context("Validate the build and deployment configuration", func() {
+	BeforeEach(func() {
+		if discovery.Enabled() && !discoverySuccessful {
+			Skip(discoveryFailedReason)
+		}
+	})
+
+	Context("Validate the build", func() {
 		It("Should forward and receive packets from a pod running dpdk base on a image created by building config", func() {
+			Expect(dpdkWorkloadPod).ToNot(BeNil(), "No dpdk workload pod found")
 			var out string
 			var err error
-			_, exist, err := findDPDKDeploymentConfigWorkloadPod()
-			Expect(err).ToNot(HaveOccurred())
 
-			if !exist {
-				Skip("skip test as we can't find a dpdk workload created by a deployment config object")
+			if dpdkWorkloadPod.Spec.Containers[0].Image == images.For(images.Dpdk) {
+				Skip("skip test as we can't find a dpdk workload running with a s2i build")
 			}
-			Expect(dpdkWorkloadPod).ToNot(BeNil(), "No dpdk workload pod found")
 
 			By("Parsing output from the DPDK application")
 			Eventually(func() string {
@@ -127,15 +167,13 @@ var _ = Describe("dpdk", func() {
 
 	Context("Validate a DPDK workload running inside a pod", func() {
 		It("Should forward and receive packets", func() {
+			Expect(dpdkWorkloadPod).ToNot(BeNil(), "No dpdk workload pod found")
 			var out string
 			var err error
-			_, exist, err := findDPDKDeploymentConfigWorkloadPod()
-			Expect(err).ToNot(HaveOccurred())
 
-			if exist {
-				Skip("skip test as we find a dpdk workload created by a deployment config object")
+			if dpdkWorkloadPod.Spec.Containers[0].Image != images.For(images.Dpdk) {
+				Skip("skip test as we find a dpdk workload running with a s2i build")
 			}
-			Expect(dpdkWorkloadPod).ToNot(BeNil(), "No dpdk workload pod found")
 
 			By("Parsing output from the DPDK application")
 			Eventually(func() string {
@@ -185,7 +223,7 @@ var _ = Describe("dpdk", func() {
 		var activeNumberOfFreeHugePages int
 		var numaNode int
 
-		execute.BeforeAll(func() {
+		BeforeEach(func() {
 			Expect(dpdkWorkloadPod).ToNot(BeNil(), "No dpdk workload pod found")
 			buff, err := pods.ExecCommand(client.Client, *dpdkWorkloadPod, []string{"cat", "/sys/fs/cgroup/cpuset/cpuset.cpus"})
 			Expect(err).ToNot(HaveOccurred())
@@ -204,8 +242,10 @@ var _ = Describe("dpdk", func() {
 		It("should allocate the amount of hugepages requested", func() {
 			Expect(activeNumberOfFreeHugePages).To(BeNumerically(">", 0))
 
+			// In case of nodeselector set, this pod will end up in a compliant node because the selection
+			// logic is applied to the workload pod.
 			pod := pods.DefineWithHugePages(namespaces.DpdkTest, dpdkWorkloadPod.Spec.NodeName)
-			pod, err := client.Client.Pods(namespaces.DpdkTest).Create(pod)
+			pod, err := client.Client.Pods(namespaces.DpdkTest).Create(context.Background(), pod, metav1.CreateOptions{})
 			Expect(err).ToNot(HaveOccurred())
 
 			Eventually(func() int {
@@ -218,15 +258,15 @@ var _ = Describe("dpdk", func() {
 				return numberOfFreeHugePages
 			}, 5*time.Minute, 5*time.Second).Should(Equal(activeNumberOfFreeHugePages - 1))
 
-			pod, err = client.Client.Pods(namespaces.DpdkTest).Get(pod.Name, metav1.GetOptions{})
+			pod, err = client.Client.Pods(namespaces.DpdkTest).Get(context.Background(), pod.Name, metav1.GetOptions{})
 			Expect(err).ToNot(HaveOccurred())
 			Expect(pod.Status.Phase).To(Equal(corev1.PodRunning))
 
-			err = client.Client.Pods(namespaces.DpdkTest).Delete(pod.Name, &metav1.DeleteOptions{GracePeriodSeconds: pointer.Int64Ptr(0)})
+			err = client.Client.Pods(namespaces.DpdkTest).Delete(context.Background(), pod.Name, metav1.DeleteOptions{GracePeriodSeconds: pointer.Int64Ptr(0)})
 			Expect(err).ToNot(HaveOccurred())
 
 			Eventually(func() error {
-				_, err := client.Client.Pods(namespaces.DpdkTest).Get(pod.Name, metav1.GetOptions{})
+				_, err := client.Client.Pods(namespaces.DpdkTest).Get(context.Background(), pod.Name, metav1.GetOptions{})
 				if err != nil && errors.IsNotFound(err) {
 					return err
 				}
@@ -239,14 +279,18 @@ var _ = Describe("dpdk", func() {
 	// This will not work if we use a random order running
 	Context("restoring configuration", func() {
 		It("should restore the cluster to the original status", func() {
-			By("restore performance profile")
-			RestorePerformanceProfile()
+			if !discovery.Enabled() {
+				By(" restore performance profile")
+				RestorePerformanceProfile()
+			}
 
 			By("cleaning the sriov test configuration")
 			CleanSriov()
 
-			By("restore sriov policies")
-			RestoreSriovPolicy()
+			if !discovery.Enabled() {
+				By("restore sriov policies")
+				RestoreSriovPolicy()
+			}
 
 			By("restore sriov networks")
 			RestoreSriovNetwork()
@@ -282,14 +326,7 @@ func getNumberOfPackets(line, firstFieldSubstr string) int {
 }
 
 func tryToFindDPDKPod() (*corev1.Pod, bool) {
-	pod, exist, err := findDPDKDeploymentConfigWorkloadPod()
-	Expect(err).ToNot(HaveOccurred())
-
-	if exist {
-		return pod, true
-	}
-
-	pod, exist, err = findDPDKWorkloadPod()
+	pod, exist, err := findDPDKWorkloadPod()
 	Expect(err).ToNot(HaveOccurred())
 
 	if exist {
@@ -299,10 +336,57 @@ func tryToFindDPDKPod() (*corev1.Pod, bool) {
 	return nil, false
 }
 
-func findOrOverridePerformanceProfile() {
-	performanceProfile, valid, err := ValidatePerformanceProfile()
-	Expect(err).ToNot(HaveOccurred())
+func discoverPerformanceProfiles() (bool, string, []*perfv1.PerformanceProfile) {
+	if enforcedPerformanceProfileName != "" {
+		performanceProfile, err := findDefaultPerformanceProfile()
+		if err != nil {
+			return false, fmt.Sprintf("Can not run tests in discovery mode. Failed to find a valid perfomance profile. %s", err), nil
+		}
+		valid, err := validatePerformanceProfile(performanceProfile)
+		if !valid || err != nil {
+			return false, fmt.Sprintf("Can not run tests in discovery mode. Failed to find a valid perfomance profile. %s", err), nil
+		}
+		return true, "", []*perfv1.PerformanceProfile{performanceProfile}
+	}
 
+	performanceProfileList := &perfv1.PerformanceProfileList{}
+	var profiles []*perfv1.PerformanceProfile
+	err := client.Client.List(context.TODO(), performanceProfileList)
+	if err != nil {
+		return false, fmt.Sprintf("Can not run tests in discovery mode. Failed to find a valid perfomance profile. %s", err), nil
+	}
+	for _, performanceProfile := range performanceProfileList.Items {
+		valid, err := validatePerformanceProfile(&performanceProfile)
+		if valid && err == nil {
+			profiles = append(profiles, &performanceProfile)
+		}
+	}
+	if len(profiles) > 0 {
+		return true, "", profiles
+	}
+	return false, fmt.Sprintf("Can not run tests in discovery mode. Failed to find a valid perfomance profile. %s", err), nil
+}
+
+func findDefaultPerformanceProfile() (*perfv1.PerformanceProfile, error) {
+	performanceProfile := &perfv1.PerformanceProfile{}
+	err := client.Client.Get(context.TODO(), goclient.ObjectKey{Name: performanceProfileName}, performanceProfile)
+	return performanceProfile, err
+}
+
+func findOrOverridePerformanceProfile() {
+	var valid = true
+	performanceProfile, err := findDefaultPerformanceProfile()
+	if err != nil {
+		if !errors.IsNotFound(err) {
+			Expect(err).ToNot(HaveOccurred())
+		}
+		valid = false
+		performanceProfile = nil
+	}
+	if valid {
+		valid, err = validatePerformanceProfile(performanceProfile)
+		Expect(err).ToNot(HaveOccurred())
+	}
 	if !valid {
 		if performanceProfile != nil {
 			OriginalPerformanceProfile = performanceProfile.DeepCopy()
@@ -317,14 +401,13 @@ func findOrOverridePerformanceProfile() {
 
 		err := CreatePerformanceProfile()
 		Expect(err).ToNot(HaveOccurred())
-
 		err = WaitForClusterToBeStable()
 		Expect(err).ToNot(HaveOccurred())
 	}
 }
 
-func findOrOverrideSriovNetwork() {
-	validSriovNetwork, err := ValidateSriovNetwork()
+func findOrOverrideSriovPolicyAndNetwork() {
+	validSriovNetwork, err := ValidateSriovNetwork(resourceName)
 	Expect(err).ToNot(HaveOccurred())
 
 	validSriovPolicy, err := ValidateSriovPolicy()
@@ -336,17 +419,20 @@ func findOrOverrideSriovNetwork() {
 
 		// Clean and create a new sriov policy and network for the dpdk application
 		CleanSriov()
-		sriovInfos, err := sriovcluster.DiscoverSriov(sriovclient, SRIOV_OPERATOR_NAMESPACE)
+		sriovInfos, err := sriovcluster.DiscoverSriov(sriovclient, namespaces.SRIOVOperator)
 		Expect(err).ToNot(HaveOccurred())
 
 		Expect(sriovInfos).ToNot(BeNil())
-		Expect(len(sriovInfos.Nodes)).To(BeNumerically(">", 0))
 
-		sriovDevice, err := findDpdkSriovDevice(sriovInfos, sriovInfos.Nodes[0])
+		nn, err := nodes.MatchingOptionalSelectorByName(sriovInfos.Nodes)
+		Expect(err).ToNot(HaveOccurred())
+		Expect(len(nn)).To(BeNumerically(">", 0))
+
+		sriovDevice, err := sriovInfos.FindOneSriovDevice(nn[0])
 		Expect(err).ToNot(HaveOccurred())
 
-		CreateSriovPolicy(sriovDevice, sriovInfos.Nodes[0], 5, "dpdknic")
-		CreateSriovNetwork(sriovDevice, "test-dpdk-network", "dpdknic")
+		CreateSriovPolicy(sriovDevice, nn[0], 5, resourceName)
+		CreateSriovNetwork(sriovDevice, "test-dpdk-network", resourceName)
 	}
 
 	// When the dpdk-testing namespace is created it takes time for the network attachment definition to be created
@@ -358,60 +444,44 @@ func findOrOverrideSriovNetwork() {
 
 }
 
-func createPod() *corev1.Pod {
-	pod, err := CreateDPDKWorkload()
-	Expect(err).ToNot(HaveOccurred())
-
-	return pod
-}
-
-func ValidatePerformanceProfile() (*perfv1alpha1.PerformanceProfile, bool, error) {
-	performanceProfile := &perfv1alpha1.PerformanceProfile{}
-	err := client.Client.Get(context.TODO(), goclient.ObjectKey{Name: performanceProfileName}, performanceProfile)
-	if err != nil {
-		if errors.IsNotFound(err) {
-			return nil, false, nil
-		}
-
-		return nil, false, err
-	}
+func validatePerformanceProfile(performanceProfile *perfv1.PerformanceProfile) (bool, error) {
 
 	// Check we have more then two isolated CPU
 	cpuSet, err := cpuset.Parse(string(*performanceProfile.Spec.CPU.Isolated))
 	if err != nil {
-		return performanceProfile, false, err
+		return false, err
 	}
 
 	cpuSetSlice := cpuSet.ToSlice()
 	if len(cpuSetSlice) < 6 {
-		return performanceProfile, false, nil
+		return false, nil
 	}
 
 	if performanceProfile.Spec.HugePages == nil {
-		return performanceProfile, false, nil
+		return false, nil
 	}
 
 	if *performanceProfile.Spec.HugePages.DefaultHugePagesSize != "1G" {
-		return performanceProfile, false, nil
+		return false, nil
 	}
 
 	if len(performanceProfile.Spec.HugePages.Pages) == 0 {
-		return performanceProfile, false, nil
+		return false, nil
 	}
 
 	if performanceProfile.Spec.HugePages.Pages[0].Count < 4 {
-		return performanceProfile, false, nil
+		return false, nil
 	}
 
 	if performanceProfile.Spec.HugePages.Pages[0].Size != "1G" {
-		return performanceProfile, false, nil
+		return false, nil
 	}
 
-	return performanceProfile, true, nil
+	return true, nil
 }
 
 func CleanPerformanceProfiles() error {
-	performanceProfileList := &perfv1alpha1.PerformanceProfileList{}
+	performanceProfileList := &perfv1.PerformanceProfileList{}
 	err := client.Client.List(context.TODO(), performanceProfileList, &goclient.ListOptions{})
 	if err != nil {
 		return err
@@ -456,21 +526,21 @@ func WaitForClusterToBeStable() error {
 }
 
 func CreatePerformanceProfile() error {
-	isolatedCPUSet := perfv1alpha1.CPUSet("8-15")
-	reservedCPUSet := perfv1alpha1.CPUSet("0-7")
-	hugepageSize := perfv1alpha1.HugePageSize("1G")
-	performanceProfile := &perfv1alpha1.PerformanceProfile{
+	isolatedCPUSet := perfv1.CPUSet("8-15")
+	reservedCPUSet := perfv1.CPUSet("0-7")
+	hugepageSize := perfv1.HugePageSize("1G")
+	performanceProfile := &perfv1.PerformanceProfile{
 		ObjectMeta: metav1.ObjectMeta{
 			Name: performanceProfileName,
 		},
-		Spec: perfv1alpha1.PerformanceProfileSpec{
-			CPU: &perfv1alpha1.CPU{
+		Spec: perfv1.PerformanceProfileSpec{
+			CPU: &perfv1.CPU{
 				Isolated: &isolatedCPUSet,
 				Reserved: &reservedCPUSet,
 			},
-			HugePages: &perfv1alpha1.HugePages{
+			HugePages: &perfv1.HugePages{
 				DefaultHugePagesSize: &hugepageSize,
-				Pages: []perfv1alpha1.HugePage{
+				Pages: []perfv1.HugePage{
 					{
 						Count: 16,
 						Size:  hugepageSize,
@@ -479,7 +549,7 @@ func CreatePerformanceProfile() error {
 				},
 			},
 			NodeSelector: map[string]string{
-				"node-role.kubernetes.io/worker-cnf": "",
+				fmt.Sprintf("node-role.kubernetes.io/%s", machineConfigPoolName): "",
 			},
 		},
 	}
@@ -487,9 +557,10 @@ func CreatePerformanceProfile() error {
 	return client.Client.Create(context.TODO(), performanceProfile)
 }
 
-func ValidateSriovNetwork() (bool, error) {
+func ValidateSriovNetwork(resourceName string) (bool, error) {
 	sriovNetwork := &sriovv1.SriovNetwork{}
-	err := client.Client.Get(context.TODO(), goclient.ObjectKey{Name: "dpdk-network", Namespace: SRIOV_OPERATOR_NAMESPACE}, sriovNetwork)
+	err := client.Client.Get(context.TODO(), goclient.ObjectKey{Name: "dpdk-network", Namespace: namespaces.SRIOVOperator}, sriovNetwork)
+
 	if err != nil {
 		if errors.IsNotFound(err) {
 			return false, nil
@@ -503,13 +574,13 @@ func ValidateSriovNetwork() (bool, error) {
 
 func ValidateSriovPolicy() (bool, error) {
 	sriovPolicies := &sriovv1.SriovNetworkNodePolicyList{}
-	err := client.Client.List(context.TODO(), sriovPolicies, &goclient.ListOptions{Namespace: SRIOV_OPERATOR_NAMESPACE})
+	err := client.Client.List(context.TODO(), sriovPolicies, &goclient.ListOptions{Namespace: namespaces.SRIOVOperator})
 	if err != nil {
 		return false, err
 	}
 
 	for _, policy := range sriovPolicies.Items {
-		if policy.Spec.ResourceName == "dpdknic" {
+		if policy.Spec.ResourceName == resourceName {
 			return true, nil
 		}
 	}
@@ -517,85 +588,11 @@ func ValidateSriovPolicy() (bool, error) {
 	return false, nil
 }
 
-// findDpdkSriovDevice will search for the default gateway interface to be used as the PF
-func findDpdkSriovDevice(enabledNodes *sriovcluster.EnabledNodes, nodeName string) (*sriovv1.InterfaceExt, error) {
-	nodeStatus, ok := enabledNodes.States[nodeName]
-	if !ok {
-		return nil, fmt.Errorf("Node %s not found", nodeName)
-	}
-
-	podList := &corev1.PodList{}
-	err := client.Client.List(context.Background(), podList, &goclient.ListOptions{Namespace: SRIOV_OPERATOR_NAMESPACE,
-		LabelSelector: labels.SelectorFromSet(labels.Set{"app": "sriov-network-config-daemon"}),
-		FieldSelector: fields.SelectorFromSet(fields.Set{"spec.nodeName": nodeName})})
-	if err != nil {
-		return nil, err
-	}
-
-	if len(podList.Items) != 1 {
-		return nil, fmt.Errorf("failed to find sriov network config daemon pod on node %s", nodeName)
-	}
-
-	pod := podList.Items[0]
-
-	buff, err := pods.ExecCommand(client.Client, pod, []string{"ip", "route"})
-	if err != nil {
-		return nil, err
-	}
-
-	iface := ""
-	// search for the default gateway row and split the device
-	// Example output: default via 10.19.32.190 dev ens1f0 proto dhcp metric 101
-	for _, line := range strings.Split(buff.String(), "\r\n") {
-		if strings.Contains(line, "default") {
-			envSplit := strings.Split(line, "dev ")
-			if len(envSplit) != 2 {
-				return nil, fmt.Errorf("failed to split the device from the route line: %s", line)
-			}
-
-			iface = strings.Split(envSplit[1], " ")[0]
-			break
-		}
-	}
-
-	if iface == "" {
-		return nil, fmt.Errorf("failed to find the default gateway device")
-	}
-
-	// we are using ovn-kubernetes shared gateway we should find the interface connected to it
-	if iface == "br-ex" {
-		buff, err := pods.ExecCommand(client.Client, pod, []string{"bridge", "link"})
-		if err != nil {
-			return nil, err
-		}
-		for _, line := range strings.Split(buff.String(), "\r\n") {
-			if strings.Contains(line, "master ovs-system") {
-				envSplit := strings.Split(line, ": ")
-				if len(envSplit) != 3 {
-					return nil, fmt.Errorf("failed to split the device from the bridge link line: %s", line)
-				}
-
-				iface = strings.Split(envSplit[1], " ")[0]
-				break
-			}
-		}
-
-	}
-
-	for _, itf := range nodeStatus.Status.Interfaces {
-		if itf.Name == iface {
-			return &itf, nil
-		}
-	}
-
-	return nil, fmt.Errorf("Unable to find sriov device on the default gateway device in node %s", nodeName)
-}
-
 func CreateSriovPolicy(sriovDevice *sriovv1.InterfaceExt, testNode string, numVfs int, resourceName string) {
 	nodePolicy := &sriovv1.SriovNetworkNodePolicy{
 		ObjectMeta: metav1.ObjectMeta{
 			GenerateName: "test-policy-",
-			Namespace:    SRIOV_OPERATOR_NAMESPACE,
+			Namespace:    namespaces.SRIOVOperator,
 		},
 		Spec: sriovv1.SriovNetworkNodePolicySpec{
 			NodeSelector: map[string]string{
@@ -620,13 +617,12 @@ func CreateSriovPolicy(sriovDevice *sriovv1.InterfaceExt, testNode string, numVf
 	if sriovDevice.Vendor == "8086" {
 		nodePolicy.Spec.DeviceType = "vfio-pci"
 	}
-
 	err := sriovclient.Create(context.Background(), nodePolicy)
 	Expect(err).ToNot(HaveOccurred())
-	waitForSRIOVStable()
+	sriov.WaitStable(sriovclient)
 
 	Eventually(func() int64 {
-		testedNode, err := sriovclient.Nodes().Get(testNode, metav1.GetOptions{})
+		testedNode, err := sriovclient.Nodes().Get(context.Background(), testNode, metav1.GetOptions{})
 		Expect(err).ToNot(HaveOccurred())
 		resNum, _ := testedNode.Status.Allocatable[corev1.ResourceName("openshift.io/"+resourceName)]
 		capacity, _ := resNum.AsInt64()
@@ -636,7 +632,7 @@ func CreateSriovPolicy(sriovDevice *sriovv1.InterfaceExt, testNode string, numVf
 
 func CreateSriovNetwork(sriovDevice *sriovv1.InterfaceExt, sriovNetworkName string, resourceName string) {
 	ipam := `{"type": "host-local","ranges": [[{"subnet": "1.1.1.0/24"}]],"dataDir": "/run/my-orchestrator/container-ipam-state"}`
-	err := sriovnetwork.CreateSriovNetwork(sriovclient, sriovDevice, sriovNetworkName, namespaces.DpdkTest, SRIOV_OPERATOR_NAMESPACE, resourceName, ipam)
+	err := sriovnetwork.CreateSriovNetwork(sriovclient, sriovDevice, sriovNetworkName, namespaces.DpdkTest, namespaces.SRIOVOperator, resourceName, ipam)
 	Expect(err).ToNot(HaveOccurred())
 	Eventually(func() error {
 		netAttDef := &sriovk8sv1.NetworkAttachmentDefinition{}
@@ -645,39 +641,38 @@ func CreateSriovNetwork(sriovDevice *sriovv1.InterfaceExt, sriovNetworkName stri
 
 }
 
-func CreateDPDKWorkload() (*corev1.Pod, error) {
+func createDPDKWorkload(nodeSelector map[string]string, resourceName, testpmdCommand string, runningTime int, isServer bool) (*corev1.Pod, error) {
 	resources := map[corev1.ResourceName]resource.Quantity{
 		corev1.ResourceName("hugepages-1Gi"): resource.MustParse("2Gi"),
 		corev1.ResourceMemory:                resource.MustParse("1Gi"),
 		corev1.ResourceCPU:                   resource.MustParse("4"),
 	}
-
 	container := corev1.Container{
 		Name:  "dpdk",
 		Image: images.For(images.Dpdk),
 		Command: []string{
 			"/bin/bash",
 			"-c",
-			`set -ex
+			fmt.Sprintf(`set -ex
 export CPU=$(cat /sys/fs/cgroup/cpuset/cpuset.cpus)
 echo ${CPU}
-echo ${PCIDEVICE_OPENSHIFT_IO_DPDKNIC}
+echo ${PCIDEVICE_OPENSHIFT_IO_%s}
 
 cat <<EOF >test.sh
-spawn testpmd -l ${CPU} -w ${PCIDEVICE_OPENSHIFT_IO_DPDKNIC} --iova-mode=va -- -i --portmask=0x1 --nb-cores=2 --forward-mode=mac --port-topology=loop --no-mlockall
+spawn %s
 set timeout 10000
 expect "testpmd>"
 send -- "port stop 0\r"
 expect "testpmd>"
 send -- "port detach 0\r"
 expect "testpmd>"
-send -- "port attach ${PCIDEVICE_OPENSHIFT_IO_DPDKNIC}\r"
+send -- "port attach ${PCIDEVICE_OPENSHIFT_IO_%s}\r"
 expect "testpmd>"
 send -- "port start 0\r"
 expect "testpmd>"
 send -- "start\r"
 expect "testpmd>"
-sleep 30
+sleep %d
 send -- "stop\r"
 expect "testpmd>"
 send -- "quit\r"
@@ -687,11 +682,14 @@ EOF
 expect -f test.sh
 
 sleep INF
-`},
+`, resourceName, testpmdCommand, resourceName, runningTime)},
 		SecurityContext: &corev1.SecurityContext{
 			RunAsUser: pointer.Int64Ptr(0),
 			Capabilities: &corev1.Capabilities{
-				Add: []corev1.Capability{"IPC_LOCK", "SYS_RESOURCE"},
+				// Enable NET_RAW is required by mellanox nics as they are using the netdevice driver
+				// NET_RAW was removed from the default capabilities
+				// https://access.redhat.com/security/cve/cve-2020-14386
+				Add: []corev1.Capability{"IPC_LOCK", "SYS_RESOURCE","NET_RAW"},
 			},
 		},
 		Env: []corev1.EnvVar{
@@ -733,9 +731,57 @@ sleep INF
 					},
 				},
 			},
-		}}
+		},
+	}
 
-	dpdkPod, err := client.Client.Pods(namespaces.DpdkTest).Create(dpdkPod)
+	if len(nodeSelector) > 0 {
+		dpdkPod.Spec.NodeSelector = nodeSelector
+	}
+
+	if nodeSelector != nil && len(nodeSelector) > 0 {
+		if dpdkPod.Spec.NodeSelector == nil {
+			dpdkPod.Spec.NodeSelector = make(map[string]string)
+		}
+		for k, v := range nodeSelector {
+			dpdkPod.Spec.NodeSelector[k] = v
+		}
+	}
+
+	imageStream, err := client.Client.ImageStreams(DEMO_APP_NAMESPACE).Get(context.TODO(), "s2i-dpdk-app", metav1.GetOptions{})
+	if err != nil {
+		if !errors.IsNotFound(err) {
+			return nil, err
+		}
+	}
+
+	if len(imageStream.Status.Tags) > 0 && !discovery.Enabled() {
+		// Use the command from the image
+		if isServer {
+			dpdkPod.Spec.Containers[0].Command = nil
+		}
+		dpdkPod.Spec.Containers[0].Image = "image-registry.openshift-image-registry.svc:5000/dpdk/s2i-dpdk-app:latest"
+
+		_, err = client.Client.RoleBindings(DEMO_APP_NAMESPACE).Get(context.TODO(), "system:image-puller", metav1.GetOptions{})
+		if err != nil {
+			if !errors.IsNotFound(err) {
+				return nil, err
+			}
+
+			//We need to create a rolebinding to allow the dpdk-testing project to pull image from the dpdk project
+			roleBind := rbacv1.RoleBinding{ObjectMeta: metav1.ObjectMeta{Name: "system:image-puller", Namespace: DEMO_APP_NAMESPACE},
+				RoleRef: rbacv1.RoleRef{Name: "system:image-puller", Kind: "ClusterRole", APIGroup: "rbac.authorization.k8s.io"},
+				Subjects: []rbacv1.Subject{
+					{Kind: "ServiceAccount", Name: "default", Namespace: namespaces.DpdkTest},
+				}}
+
+			_, err = client.Client.RoleBindings(DEMO_APP_NAMESPACE).Create(context.TODO(), &roleBind, metav1.CreateOptions{})
+			if err != nil {
+				return nil, err
+			}
+		}
+	}
+
+	dpdkPod, err = client.Client.Pods(namespaces.DpdkTest).Create(context.Background(), dpdkPod, metav1.CreateOptions{})
 	if err != nil {
 		return nil, err
 	}
@@ -753,11 +799,6 @@ sleep INF
 	return dpdkPod, nil
 }
 
-// findDPDKDeploymentConfigWorkloadPod finds a pod running a DPDK application from a deployment config
-func findDPDKDeploymentConfigWorkloadPod() (*corev1.Pod, bool, error) {
-	return findDPDKWorkloadPodByLabelSelector(labels.SelectorFromSet(labels.Set{"deploymentconfig": "s2i-dpdk-app"}).String(), DEMO_APP_NAMESPACE)
-}
-
 // findDPDKWorkloadPod finds a pod running a DPDK application using a know label
 // Label: app="dpdk"
 func findDPDKWorkloadPod() (*corev1.Pod, bool, error) {
@@ -769,7 +810,7 @@ func findDPDKWorkloadPodByLabelSelector(labelSelector, namespace string) (*corev
 		LabelSelector: labelSelector,
 	}
 
-	p, err := client.Client.Pods(namespace).List(listOptions)
+	p, err := client.Client.Pods(namespace).List(context.Background(), listOptions)
 	if err != nil {
 		return nil, false, err
 	}
@@ -880,8 +921,9 @@ func findNUMAForSRIOV(pod *corev1.Pod) (int, error) {
 	buff, err := pods.ExecCommand(client.Client, *pod, []string{"env"})
 	Expect(err).ToNot(HaveOccurred())
 	pciAddress := ""
+	pciEnvVariableName := fmt.Sprintf("PCIDEVICE_OPENSHIFT_IO_%s", strings.ToUpper(resourceName))
 	for _, line := range strings.Split(buff.String(), "\r\n") {
-		if strings.Contains(line, PCI_ENV_VARIABLE_NAME) {
+		if strings.Contains(line, pciEnvVariableName) {
 			envSplit := strings.Split(line, "=")
 			Expect(len(envSplit)).To(Equal(2))
 			pciAddress = envSplit[1]
@@ -900,12 +942,12 @@ func findNUMAForSRIOV(pod *corev1.Pod) (int, error) {
 			return pciNuma, err
 		}
 	}
-	return -1, fmt.Errorf("failed to find the numa for pci %s", PCI_ENV_VARIABLE_NAME)
+	return -1, fmt.Errorf("failed to find the numa for pci %s", pciEnvVariableName)
 }
 
 func BackupSriovPolicy() {
 	sriovPolicyList := &sriovv1.SriovNetworkNodePolicyList{}
-	err := sriovclient.List(context.TODO(), sriovPolicyList, &goclient.ListOptions{Namespace: SRIOV_OPERATOR_NAMESPACE})
+	err := sriovclient.List(context.TODO(), sriovPolicyList, &goclient.ListOptions{Namespace: namespaces.SRIOVOperator})
 	Expect(err).ToNot(HaveOccurred())
 
 	for _, policy := range sriovPolicyList.Items {
@@ -924,7 +966,7 @@ func BackupSriovPolicy() {
 
 	Eventually(func() bool {
 		toCheck := &sriovv1.SriovNetworkNodePolicyList{}
-		err := sriovclient.List(context.TODO(), toCheck, &goclient.ListOptions{Namespace: SRIOV_OPERATOR_NAMESPACE})
+		err := sriovclient.List(context.TODO(), toCheck, &goclient.ListOptions{Namespace: namespaces.SRIOVOperator})
 		Expect(err).ToNot(HaveOccurred())
 		return (len(toCheck.Items) == 1 && toCheck.Items[0].Name == "default")
 	}, 1*time.Minute, 1*time.Second).Should(BeTrue())
@@ -932,7 +974,7 @@ func BackupSriovPolicy() {
 
 func BackupSriovNetwork() {
 	sriovNetworkList := &sriovv1.SriovNetworkList{}
-	err := sriovclient.List(context.TODO(), sriovNetworkList, &goclient.ListOptions{Namespace: SRIOV_OPERATOR_NAMESPACE})
+	err := sriovclient.List(context.TODO(), sriovNetworkList, &goclient.ListOptions{Namespace: namespaces.SRIOVOperator})
 	Expect(err).ToNot(HaveOccurred())
 
 	for _, network := range sriovNetworkList.Items {
@@ -947,7 +989,7 @@ func BackupSriovNetwork() {
 
 	Eventually(func() int {
 		toCheck := &sriovv1.SriovNetworkList{}
-		err := sriovclient.List(context.TODO(), toCheck, &goclient.ListOptions{Namespace: SRIOV_OPERATOR_NAMESPACE})
+		err := sriovclient.List(context.TODO(), toCheck, &goclient.ListOptions{Namespace: namespaces.SRIOVOperator})
 		Expect(err).ToNot(HaveOccurred())
 		return len(toCheck.Items)
 	}, 1*time.Minute, 1*time.Second).Should(Equal(0))
@@ -976,17 +1018,17 @@ func RestorePerformanceProfile() {
 func RestoreSriovPolicy() {
 	for _, policy := range OriginalSriovPolicies {
 		name := policy.Name
-		policy.ObjectMeta = metav1.ObjectMeta{Name: name, Namespace: SRIOV_OPERATOR_NAMESPACE}
+		policy.ObjectMeta = metav1.ObjectMeta{Name: name, Namespace: namespaces.SRIOVOperator}
 		err := sriovclient.Create(context.TODO(), policy)
 		Expect(err).ToNot(HaveOccurred())
 	}
-	waitForSRIOVStable()
+	sriov.WaitStable(sriovclient)
 }
 
 func RestoreSriovNetwork() {
 	for _, network := range OriginalSriovNetworks {
 		name := network.Name
-		network.ObjectMeta = metav1.ObjectMeta{Name: name, Namespace: SRIOV_OPERATOR_NAMESPACE}
+		network.ObjectMeta = metav1.ObjectMeta{Name: name, Namespace: namespaces.SRIOVOperator}
 		err := sriovclient.Create(context.TODO(), network)
 		Expect(err).ToNot(HaveOccurred())
 		Eventually(func() error {
@@ -998,26 +1040,13 @@ func RestoreSriovNetwork() {
 
 func CleanSriov() {
 	// This clean only the policy and networks with the prefix of test
-	err := sriovnamespaces.Clean(SRIOV_OPERATOR_NAMESPACE, namespaces.DpdkTest, sriovclient)
+	err := sriovnamespaces.CleanPods(namespaces.DpdkTest, sriovclient)
 	Expect(err).ToNot(HaveOccurred())
-	waitForSRIOVStable()
-}
-
-func waitForSRIOVStable() {
-	// This used to be to check for sriov not to be stable first,
-	// then stable. The issue is that if no configuration is applied, then
-	// the status won't never go to not stable and the test will fail.
-	// TODO: find a better way to handle this scenario
-	time.Sleep(5 * time.Second)
-	Eventually(func() bool {
-		res, err := sriovcluster.SriovStable(SRIOV_OPERATOR_NAMESPACE, sriovclient)
+	if !discovery.Enabled() {
+		err = sriovnamespaces.CleanPolicies(namespaces.SRIOVOperator, sriovclient)
 		Expect(err).ToNot(HaveOccurred())
-		return res
-	}, waitingTime, 1*time.Second).Should(BeTrue())
-
-	Eventually(func() bool {
-		isClusterReady, err := sriovcluster.IsClusterStable(sriovclient)
-		Expect(err).ToNot(HaveOccurred())
-		return isClusterReady
-	}, waitingTime, 1*time.Second).Should(BeTrue())
+	}
+	err = sriovnamespaces.CleanNetworks(namespaces.SRIOVOperator, sriovclient)
+	Expect(err).ToNot(HaveOccurred())
+	sriov.WaitStable(sriovclient)
 }
